/**
 * newPost post a new entry to metaWeblog api compatible server
 * returns the unique postId assigned
 */
function metaWeblogNewPost(postTitle, postContent, postPublish)
{
  initBlog();
  editorHandle.currentStatus("Making new posts...", null);
  editorHandle.viewStatus();
  var content      = xmlRpc.createType(xmlRpc.STRUCT, {});
  var title        = xmlRpc.createType(xmlRpc.STRING, {});
  var description  = xmlRpc.createType(xmlRpc.STRING, {});
  var category     = xmlRpc.createType(xmlRpc.ARRAY, {});
  var catEntry1    = xmlRpc.createType(xmlRpc.STRING, {});
  var publish      = xmlRpc.createType(xmlRpc.BOOLEAN, {});

  title.data       = postTitle
  description.data = postContent;

  var categoryEntry = document.getElementById("categoryBox").selectedItem;
  if (categoryEntry) {
    catEntry1.data   = categoryEntry.value;
    category.AppendElement(catEntry1);
    content.setValue("categories", category);
  }

  content.setValue("title", title);
  content.setValue("description", description);

  publish.data     = postPublish;
  xmlRpc.init(serverurl);
  xmlRpc.asyncCall(metaWeblogNewPostListener, null, "metaWeblog.newPost",
                   [blogid, username, password, content, publish], 5);
}

var metaWeblogNewPostListener = {
  onResult: function(client, ctxt, result) {
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    result = result.QueryInterface(Components.interfaces.nsISupportsPrimitive);
    if (result.type == 3 || result.type == 2) {
    var blogStatus;
      try {
        blogStatus = result.QueryInterface(Components.interfaces.nsISupportsString);
      } catch (e) {
        blogStatus = result.QueryInterface(Components.interfaces.nsISupportsCString);
      }
      editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.Done"), 100);
      postStore.published = "Y";
      postStore.postID = blogStatus.data;
      postStore.saveToDisk();
      if (getAccountType() == "movabletype") {
        movableTypePublishPost(blogStatus);
      } else {
        editorHandle.viewHtml();
      }
      updateSelection();
    } else {
      editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.BlogServerRetUnexpctVal") + result.type + "\n");
    }
  }, onFault: function(client, ctxt, fault) {
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    alert(MozBlogLocalStr.getString("MozBlog.Fault") + fault + "\n");
  }, onError: function(client, ctxt, status, errorMsg) {
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    alert(MozBlogLocalStr.getString("MozBlog.Error") + status.toString(16) + ") " + errorMsg + "\n");
  }
};

/**
 * metaWeblogEditPost edit an existing entry to metaWeblog api compatible server
 * returns the true or false
 */
function metaWeblogEditPost(postID, postTitle, postContent, postPublish)
{
  initBlog();
  var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
  editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.EditPosts"), null);
  editorHandle.viewStatus();
  var content      = xmlRpc.createType(xmlRpc.STRUCT, {});
  var postId       = xmlRpc.createType(xmlRpc.STRING, {});
  var title        = xmlRpc.createType(xmlRpc.STRING, {});
  var description  = xmlRpc.createType(xmlRpc.STRING, {});
  var category     = xmlRpc.createType(xmlRpc.ARRAY, {});
  var catEntry1    = xmlRpc.createType(xmlRpc.STRING, {});
  var publish      = xmlRpc.createType(xmlRpc.BOOLEAN, {});

  postId.data      = postID;
  title.data       = postTitle
  description.data = postContent;

  var categoryEntry = document.getElementById("categoryBox").selectedItem;
  if (categoryEntry) {
    catEntry1.data   = categoryEntry.value;
    category.AppendElement(catEntry1);
    content.setValue("categories", category);
  }

  content.setValue("title", title);
  content.setValue("description", description);

  publish.data     = postPublish;
  xmlRpc.init(serverurl);
  xmlRpc.asyncCall(metaWeblogEditPostListener, null, "metaWeblog.editPost",
                   [postId, username, password, content, publish], 5);
}

var metaWeblogEditPostListener = {
  onResult: function(client, ctxt, result) {
    result = result.QueryInterface(Components.interfaces.nsISupportsPrimitive);
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    if (result.type == 4) {
      editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.Done"), 100);
      var blogStatus = result.QueryInterface(Components.interfaces.nsISupportsPRBool);
      blogStatus = blogStatus.data;
      postStore.saveToDisk();
      if (getAccountType() == "movabletype") {
        movableTypePublishPost(globalState.postID);
      } else {
        editorHandle.viewHtml();
      }
      updateSelection();
    } else {
      editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.BlogServerRetUnexpctVal") + result.type + "\n");
    }
  }, onFault: function(client, ctxt, fault) {
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    alert(MozBlogLocalStr.getString("MozBlog.Fault") + fault + "\n");
  }, onError: function(client, ctxt, status, errorMsg) {
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    alert(MozBlogLocalStr.getString("MozBlog.Error") + status.toString(16) + ") " + errorMsg + "\n");
  }
};

/**
 * getRecentPost get an array of recent post (<20)
 * return an array of dictionary with postid, content, userid, dateCreated
 */
function metaWeblogGetRecentPosts(numPost)
{
  initBlog();
  var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
  //var IntNumPost = numPost;
  //getFormattedString("MozBlog.Get", [IntNumPost]), null);
  editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.Get") + " " + numPost + " " +
                             MozBlogLocalStr.getString("MozBlog.RecentPosts"), null);
  var postNum = xmlRpc.createType(xmlRpc.INT, {});
  postNum.data = numPost;
  xmlRpc.init(serverurl);
  xmlRpc.asyncCall(metaWeblogRecentPostListener, null, "metaWeblog.getRecentPosts",
	  			  [blogid, username, password, postNum], 4);
}


var metaWeblogRecentPostListener = {
  onResult: function(client, ctxt, result) {
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    newBlogList = document.getElementById("prevBlogPostList");
    recentResult = result.QueryInterface(Components.interfaces.nsISupportsArray);
    for (i = 0; i < recentResult.Count(); i++) {
      dictResult = recentResult.QueryElementAt(i, Components.interfaces.nsIDictionary);
      dictResult1 = dictResult.getValue("dateCreated")
                              .QueryInterface(Components.interfaces.nsISupportsPRTime);
      try {
        dictResult2 = dictResult.getValue("postid")
                                .QueryInterface(Components.interfaces.nsISupportsString);
        blogContent = dictResult.getValue("description")
                                .QueryInterface(Components.interfaces.nsISupportsString);
        blogTitle   = dictResult.getValue("title")
                                .QueryInterface(Components.interfaces.nsISupportsString);
      } catch (e) {
        dictResult2 = dictResult.getValue("postid")
                                .QueryInterface(Components.interfaces.nsISupportsCString);
        blogContent = dictResult.getValue("description")
                                .QueryInterface(Components.interfaces.nsISupportsCString);
        blogTitle   = dictResult.getValue("title")
                                .QueryInterface(Components.interfaces.nsISupportsCString);
      }

      postStore.content     = blogContent;
      postStore.postID      = dictResult2;
      postStore.blogID      = document.getElementById("blogIDs").selectedItem.value;
      postStore.date        = getISO8601Date(dictResult1.data);
      postStore.filename    = dictResult1.data + ".html";
      postStore.title       = blogTitle;
      postStore.description = null;
      postStore.category    = null;
      postStore.permLink    = null;
      postStore.published   = "Y";
      postStore.saveToDisk();
    }
    editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.PostsImported"), 100);
    clearField();
    /*if (recentPublish) {
      pingSelectedBlog();
      recentPublish = false;
    }*/
  }, onFault: function(client, ctxt, fault) {
    dump("Fault! " + fault + "\n");
  }, onError: function(client, ctxt, status, errorMsg) {
    dump("Error! (" + status.toString(16) + ") " + errorMsg + "\n");
  }
};

/**
 * mt.publishPost publish the movabletype by rebuilding the index
 * return boolean
 */
function movableTypePublishPost(postID)
{
  initBlog();
  var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
  editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.PublishPost"), null);
  var postId       = xmlRpc.createType(xmlRpc.STRING, {});
  postId.data      = postID;
  xmlRpc.init(serverurl);
  xmlRpc.asyncCall(movableTypePublishPostListener, null, "mt.publishPost",
	  			  [postId, username, password], 3);
}

var movableTypePublishPostListener = {
  onResult: function(client, ctxt, result) {
    result = result.QueryInterface(Components.interfaces.nsISupportsPrimitive);
    var MozBlogLocalStr = document.getElementById("bundle_MozBlog");
    if (result.type == 4) {
      var blogStatus = result.QueryInterface(Components.interfaces.nsISupportsPRBool);
      blogStatus = blogStatus.data;
      editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.Done"), 100);
      editorHandle.viewHtml();
    } else {
      editorHandle.currentStatus(MozBlogLocalStr.getString("MozBlog.BlogServerRetUnexpctVal") + result.type + "\n");
    }
  }, onFault: function(client, ctxt, fault) {
    dump("Fault! " + fault + "\n");
  }, onError: function(client, ctxt, status, errorMsg) {
    dump("Error! (" + status.toString(16) + ") " + errorMsg + "\n");
  }
};